Method: Scale#clear_scale

Defined in:
lib/scale.rb

#clear_scaleObject

You’ll need to call this function only if you’re planning to draw a second chart in the rendered picture. Calling this function will clear the current scaling parameters thus you’ll need to call again the draw_scale function before drawing any new chart.



5
6
7
8
9
10
11
12
# File 'lib/scale.rb', line 5

def clear_scale
  @vmin     = nil
  @vmax       = nil
  @v_x_min   = nil
  @v_x_max      = nil
  @divisions  = 0
  @x_divisions = 0
end